{$I HC.INC}  { Standard defines }

unit HCFormRg;

interface

uses

  Windows, SysUtils, StdCtrls, HCForms, Consts, ExptIntf, EditIntf;

const
  sHCFormObjRepositoryPage = 'Pointer';
  sHCFormProjObjRepositoryPage = 'Pointer';

type

  THCFormExpert = class(TIExpert)
  public
    function GetStyle: TExpertStyle; override;
    function GetName: string; override;
    function GetAuthor: string; override;
    function GetComment: string; override;
    function GetPage: string; override;
    function GetGlyph: HICON; override;
    function GetState: TExpertState; override;
    function GetIDString: string; override;
    function GetMenuText: string; override;
    procedure Execute; override;
  end;

  THCFormProjectExpert = class(TIExpert)
  public
    function GetStyle: TExpertStyle; override;
    function GetName: string; override;
    function GetAuthor: string; override;
    function GetComment: string; override;
    function GetPage: string; override;
    function GetGlyph: HICON; override;
    function GetState: TExpertState; override;
    function GetIDString: string; override;
    function GetMenuText: string; override;
    procedure Execute; override;
  end;

  THCFormProjectCreator = class(TIProjectCreator)
  public
    function Existing: boolean; override;
    function GetFileName: string; override;
    function GetFileSystem: string; override;
    function NewProjectSource(const ProjectName: string): string; override;
    procedure NewDefaultModule; override;
    procedure NewProjectResource(Module: TIModuleInterface); override;
  end;

  procedure Register;


implementation

uses
  ToolIntf, DsgnIntf, TypInfo;


const
  CRLF = #13#10;


procedure Register;
begin
  RegisterCustomModule(THCForm, TCustomModule);
  RegisterLibraryExpert(THCFormExpert.Create);
  RegisterLibraryExpert(THCFormProjectExpert.Create);
end;

type
  {$IFDEF HC_DELPHI_3}
  THCFormModuleCreator = class(TIModuleCreator)
  {$ELSE}
  THCFormModuleCreator = class(TIModuleCreatorEx)
  {$ENDIF}
  private
    FAncestorIdent: string;
    FAncestorClass: TClass;
    FFormIdent: string;
    FUnitIdent: string;
    FFileName: string;
  public
    function Existing: boolean; override;
    function GetFileName: string; override;
    function GetFileSystem: string; override;
    function GetFormName: string; override;
    function GetAncestorName: string; override;
    function NewModuleSource({$IFNDEF HC_DELPHI_3} const {$ENDIF} UnitIdent,
       FormIdent, AncestorIdent: string): string; override;
    {$IFNDEF HC_DELPHI_3}
    function GetIntfName: string; override;
    function NewIntfSource(const UnitIdent,
       FormIdent,AncestorIdent: string): string; override;
    {$ENDIF}
    procedure FormCreated(Form: TIFormInterface); override;
  end;


{ THCFormModuleCreator }

function THCFormModuleCreator.Existing: boolean;
begin
  Result:= FALSE;
end;

function THCFormModuleCreator.GetFileName: string;
begin
  Result:= '';
end;

function THCFormModuleCreator.GetFileSystem: string;
begin
  Result:= '';
end;

function THCFormModuleCreator.GetFormName: string;
begin
  Result:= FFormIdent;
end;

function THCFormModuleCreator.GetAncestorName: string;
begin
  Result:= FAncestorIdent;
end;

{$IFDEF HC_CPPB}

function UnitName2NameSpace(const Value: string): string;
var
  s1, s2: string;
begin
  Result:= '';
  if Value <> '' then
  begin
    s1:= Value[1];
    s2:= LowerCase(Value);
    System.Delete(s2, 1, 1);
    Result:= UpperCase(s1)+s2;
  end;
end;

{$ENDIF}

function GetCustomFormUnit(const AClass: TClass): string;
begin
  Result:= GetTypeData(PTypeInfo(AClass.ClassInfo))^.UnitName;
end;

{$IFNDEF HC_DELPHI_3}

function THCFormModuleCreator.GetIntfName: string;
begin
  Result:= '';
end;

const
  COMMENT_LINE = '//---------------------------------------------------------------------------' + CRLF;

function THCFormModuleCreator.NewIntfSource(const UnitIdent, FormIdent,
   AncestorIdent: string): string;
begin
  {$IFDEF HC_CPPB}
  Result := COMMENT_LINE +
     '#ifndef ' + UnitIdent + 'H' + CRLF +
     '#define ' + UnitIdent + 'H' + CRLF +
     COMMENT_LINE +
     '#include <Classes.hpp>' + CRLF +
     '#include <Controls.hpp>' + CRLF +
     '#include <StdCtrls.hpp>' + CRLF +
     '#include <Forms.hpp>' + CRLF ;

  if (AncestorIdent <> 'Form') and (FAncestorIdent <> 'DataModule') then
    Result := Result + '#include "' +
       GetCustomFormUnit(FAncestorClass) + '.hpp"' + CRLF;

  Result := Result + COMMENT_LINE +
     'class T' + FormIdent + ' : public ' + FAncestorClass.ClassName + CRLF +
     '{' + CRLF +
      '__published: // IDE-managed Components' + CRLF +
      'private: // User declarations' + CRLF +
      'protected: // User declarations' + CRLF +
      'public: // User declarations' + CRLF +
      '  __fastcall T' + FormIdent + '(TComponent* Owner);' + CRLF +
      '__published: // User declarations' + CRLF +
      '};' + CRLF + COMMENT_LINE +
      'extern PACKAGE T'+FormIdent+' *'+FormIdent+';' + CRLF + COMMENT_LINE +
      '#endif';
  {$ELSE}
  Result := ''; // Delphi doesn't use this
  {$ENDIF}
end;

{$ENDIF}

function THCFormModuleCreator.NewModuleSource(
   {$IFNDEF HC_DELPHI_3} const {$ENDIF} UnitIdent, FormIdent,
   AncestorIdent: string): string;
begin
  {$IFDEF HC_CPPB}
  Result := COMMENT_LINE + '#include <vcl.h>' + CRLF +
     '#pragma hdrstop' + CRLF + CRLF +
     '#include "' + UnitIdent + '.h"' + CRLF + COMMENT_LINE +
     '#pragma package(smart_init)' + CRLF;

  if (FAncestorIdent <> 'Form') and (FAncestorIdent <> 'DataModule') then
    Result := Result + '#pragma link "' +
       GetCustomFormUnit(FAncestorClass) + '"' + CRLF;

  Result := Result + '#pragma resource "*.dfm"' + CRLF +
     'T' + FormIdent + ' *' + FormIdent+';' + CRLF + COMMENT_LINE +
     '__fastcall T' + FormIdent + '::T' + FormIdent + '(TComponent* Owner)' + CRLF +
     '        : ' + FAncestorClass.ClassName + '(Owner)' + CRLF +
     '{' + CRLF +
     '}' + CRLF + COMMENT_LINE;
  {$ELSE}
  Result := 'unit ' + FUnitIdent + ';' + CRLF + CRLF +
     'interface' + CRLF + CRLF +
     'uses' + CRLF +
     '  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs';

  if (FAncestorIdent <> 'Form') and (FAncestorIdent <> 'DataModule') then
    Result := Result + ',' + CRLF +
       '  ' + GetCustomFormUnit(FAncestorClass);

  Result := Result + ';' + CRLF + CRLF +
     'type' + CRLF +
     '  T' + FFormIdent + ' = class(' + FAncestorClass.ClassName + ')' + CRLF +
     '  private' + CRLF +
     '    { Private declarations }' + CRLF +
     '  protected' + CRLF +
     '    { Protected declarations }' + CRLF +
     '  public' + CRLF +
     '    { Public declarations }' + CRLF +
     '  published' + CRLF +
     '    { Published declarations }' + CRLF +
     '  end;' + CRLF + CRLF +
     'var' + CRLF +
     '  ' + FFormIdent + ' : T' + FFormIdent + ';' + CRLF + CRLF +
     'implementation' + CRLF + CRLF +
     '{$R *.DFM}' + CRLF + CRLF +
     'end.' + CRLF;
  {$ENDIF}
end;

procedure THCFormModuleCreator.FormCreated(Form: TIFormInterface);
begin
  // do nothing
end;

{ THCFormExpert }

function THCFormExpert.GetStyle: TExpertStyle;
begin
  // Make it show up in the object repository (File | New)
  Result:= esForm;
end;

function THCFormExpert.GetName: String;
begin
  // official name
  Result:= 'HC Form'
end;

function THCFormExpert.GetAuthor: string;
begin
  Result:= 'Attila Szomor';
end;

function THCFormExpert.GetComment: String;
begin
  Result:= 'Create a new HCForm in current project';
end;

function THCFormExpert.GetPage: string;
begin
  Result:= sHCFormObjRepositoryPage;
end;

function THCFormExpert.GetGlyph: HICON;
begin
  Result:= LoadIcon(hInstance, 'THCFORM');
end;

function THCFormExpert.GetState: TExpertState;
begin
  // not used in a esForm expert
  Result:= [esEnabled];
end;

function THCFormExpert.GetIDString: String;
begin
  // must be unique
  Result:= 'THCFormWizard';
end;

function THCFormExpert.GetMenuText: string;
begin
  Result:= ''; // not used for esForm, just here to shut up the compiler warning.
end;

procedure THCFormExpert.Execute;
var
  IModuleCreator: THCFormModuleCreator;
  IModule: TIModuleInterface;
begin
  IModuleCreator:= THCFormModuleCreator.Create;
  try
    IModuleCreator.FAncestorIdent:= 'HCForm'; // Don't include the 'T'!!!!
    IModuleCreator.FAncestorClass:= THCForm;
    ToolServices.GetNewModuleAndClassName(IModuleCreator.FAncestorIdent,
    IModuleCreator.FUnitIdent,IModuleCreator.FFormIdent,IModuleCreator.FFileName);
    {$IFDEF HC_DELPHI_3}
    IModule:= ToolServices.ModuleCreate(IModuleCreator, [cmShowSource,
       cmShowForm, cmMarkModified, cmAddToProject, cmUnNamed]);
    {$ELSE}
    IModule:= ToolServices.ModuleCreateEx(IModuleCreator, [cmShowSource,
       cmShowForm, cmMarkModified, cmAddToProject, cmUnNamed]);
    {$ENDIF}
    IModule.Free;
  finally
    IModuleCreator.Free;
  end;
end;



{ THCFormProjectExpert }

function THCFormProjectExpert.GetStyle: TExpertStyle;
begin
  // Make it show up in the object repository (File | New)
  Result:= esProject;
end;

function THCFormProjectExpert.GetName: String;
begin
  // official name
  Result:= 'HC Form Application'
end;

function THCFormProjectExpert.GetAuthor: string;
begin
  Result:= 'Attila Szomor';
end;

function THCFormProjectExpert.GetComment: String;
begin
  Result:= 'New project with HCForm as main form';
end;

function THCFormProjectExpert.GetPage: string;
begin
  Result:= sHCFormProjObjRepositoryPage;
end;

function THCFormProjectExpert.GetGlyph: HICON;
begin
  Result:= LoadIcon(hInstance, 'THCFORM');
end;

function THCFormProjectExpert.GetState: TExpertState;
begin
  // not used in a esForm expert
  Result:= [esEnabled];
end;

function THCFormProjectExpert.GetIDString: String;
begin
  // must be unique
  Result:= 'THCFormProjectWizard';
end;

function THCFormProjectExpert.GetMenuText: string;
begin
  Result:= ''; // not used for esForm, just here to shut up the compiler warning.
end;

procedure THCFormProjectExpert.Execute;
var
  ModIntf: TIModuleInterface;
  ProjCreator: THCFormProjectCreator;
begin
  ProjCreator := THCFormProjectCreator.Create;
  ModIntf:= ToolServices.ProjectCreate(ProjCreator, [cpApplication, cpCanShowSource]);
  ModIntf.Free;
  ProjCreator.Free;
end;

{ THCFormProjectCreator }

function THCFormProjectCreator.Existing: boolean;
begin
  Result:= FALSE;
end;

function THCFormProjectCreator.GetFileName: string;
begin
  Result:= '';
end;

function THCFormProjectCreator.GetFileSystem: string;
begin
  Result:= '';
end;

procedure THCFormProjectCreator.NewDefaultModule;
var
  IModuleCreator: THCFormModuleCreator;
  IModule: TIModuleInterface;
begin
  IModuleCreator:= THCFormModuleCreator.Create;
  try
    IModuleCreator.FAncestorIdent:= 'HCForm'; // Don't include the 'T'!!!!
    IModuleCreator.FAncestorClass:= THCForm;
    ToolServices.GetNewModuleAndClassName(IModuleCreator.FAncestorIdent,
    IModuleCreator.FUnitIdent,IModuleCreator.FFormIdent,IModuleCreator.FFileName);
    {$IFDEF HC_DELPHI_3}
    IModule:= ToolServices.ModuleCreate(IModuleCreator, [cmShowSource,
       cmShowForm, cmMainForm, cmMarkModified, cmAddToProject, cmUnNamed]);
    {$ELSE}
    IModule:= ToolServices.ModuleCreateEx(IModuleCreator, [cmShowSource,
       cmShowForm, cmMainForm, cmMarkModified, cmAddToProject, cmUnNamed]);
    {$ENDIF}
    IModule.Free;
  finally
    IModuleCreator.Free;
  end;
end;

procedure THCFormProjectCreator.NewProjectResource(Module: TIModuleInterface);
begin
  Module.Free;
end;

function THCFormProjectCreator.NewProjectSource(const ProjectName: string): string;
begin
  Result:= Format('program %s;' + CRLF + CRLF, [ProjectName]) +
     'uses' + CRLF +
     '  Forms;' + CRLF + CRLF +
     '{$R *.RES}' + CRLF + CRLF +
     'begin' + CRLF +
     '  Application.Initialize;' + CRLF +
     '  Application.Run;' + CRLF +
     'end.' + CRLF;
end;


end.
